import Peer from "peerjs";
import { readHello } from "./protocol/hello";
import { BinaryStreamReader } from "./binary/readerStream";

export class Client {
  static async connect() {
    const transport = new WebTransport(`https://${location.hostname}:4433`, { allowPooling: false });
    await transport.ready;
    const stream = await transport.createBidirectionalStream();

    const peer = new Peer();
    await new Promise((res) => {
      peer.on("open", () => {
        console.log("open!", peer.id)
        res(void 0);
      });
    });

    await stream.writable.getWriter().write(new TextEncoder().encode(peer.id));
    const reader = BinaryStreamReader.fromReader(stream.readable.getReader());
    const players = $state(await readHello(reader));

    return new Client(players, peer, transport, stream);
  }

  private constructor(
    public players: Player[],
    private peer: Peer,
    private transport: WebTransport,
    private stream: WebTransportBidirectionalStream
  ) {


  }

  get id() {
    return this.peer.id;
  }

  closed() {
    // return transport.closed;
  }
}

export class Player {
  constructor(public name: string, public id: number, public position: number[], public stage: string) { }


}
